@@ -30,7 +30,6 @@
 void  menubar_setup          (BonoboUIComponent  *uic,
 			      GtkHTMLControlData *cd);
 void  menubar_update_format  (GtkHTMLControlData *cd);
-void  menubar_set_languages  (GtkHTMLControlData *cd,
-			      const gchar        *lstr);
+void  menubar_set_languages  (GtkHTMLControlData *cd);
 void  insert_table           (GtkHTMLControlData *cd);
 #endif /* _MENUBAR_H */
